#pragma once

#include <filesystem>

#include <RobotAPI/libraries/armem/core/base/EntityBase.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>

#include "EntitySnapshot.h"



namespace armarx::armem::d_ltm
{
    /**
     * @brief An entity over a period of time.
     *
     * An entity should be a physical thing or abstract concept existing
     * (and potentially evolving) over some time.
     *
     * Examples are:
     * - objects (the green box)
     * - agents (robot, human)
     * - locations (frige, sink)
     * - grasp affordances (general, or for a specific object)
     * - images
     * - point clouds
     * - other sensory values
     *
     * At each point in time (`EntitySnapshot`), the entity can have a
     * (potentially variable) number of instances (`EntityInstance`),
     * each containing a single `AronData` object of a specific `AronType`.
     */
    class Entity :
        public base::EntityBase<EntitySnapshot, Entity>
    {
        using Base = base::EntityBase<EntitySnapshot, Entity>;

    public:

        using Base::EntityBase;


        // Conversion
        wm::Entity convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const;

        // Filesystem connection
        void reload(const std::shared_ptr<std::filesystem::path>&);
        void append(const wm::Entity&);

    private:
        std::filesystem::path _fullPath() const;
        std::filesystem::path _fullPath(const std::filesystem::path&) const;

    public:
        std::shared_ptr<std::filesystem::path> path;
    };

}
