#include "EntityInstance.h"

#include <RobotAPI/libraries/armem/core/error.h>
#include <RobotAPI/libraries/armem/core/wm/aron_conversions.h>
#include <RobotAPI/libraries/armem/core/wm/json_conversions.h>

#include <RobotAPI/libraries/aron/core/navigator/data/container/Dict.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <SimoxUtility/json/json.hpp>

#include <iostream>
#include <fstream>


namespace armarx::armem::d_ltm
{

    bool EntityInstance::equalsDeep(const EntityInstance& other) const
    {
        return id() == other.id();
    }



    void EntityInstance::update(const EntityUpdate& update)
    {
        ARMARX_CHECK_FITS_SIZE(this->index(), update.instancesData.size());
    }


    std::filesystem::path EntityInstance::_fullPath() const
    {
        if (path)
        {
            return _fullPath(*path);
        }
        ARMARX_WARNING << "The path of the disk memory instance with id '" << id().str() << "' is not set. This may lead to errors.";
        return std::filesystem::path();
    }

    std::filesystem::path EntityInstance::_fullPath(const std::filesystem::path& path) const
    {
        return path / id().memoryName / id().coreSegmentName / id().providerSegmentName / id().entityName / std::to_string(id().timestamp.toMicroSeconds()) / std::to_string(id().instanceIndex);
    }

    wm::EntityInstance EntityInstance::convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const
    {
        std::filesystem::path p = _fullPath(); // here we assume that "reload" has been called first
        std::filesystem::path d = p / (std::string(DATA_FILENAME) + ".json");

        if (std::filesystem::is_regular_file(d))
        {
            std::ifstream ifs(d);
            std::string file_content((std::istreambuf_iterator<char>(ifs)), (std::istreambuf_iterator<char>()));

            nlohmann::json j = nlohmann::json::parse(file_content);
            auto aron = std::make_shared<aron::datanavigator::DictNavigator>();
            to_aron(aron, j, expectedStructure);
            wm::EntityInstance e(id());
            from_aron(aron, e);
            return e;
        }
        else
        {
            throw error::ArMemError("An diskMemory EntityInstance is not leading to a regular file. The path was: " + d.string());
        }
    }

    void EntityInstance::reload(const std::shared_ptr<std::filesystem::path>& p_ptr)
    {
        if (!p_ptr)
        {
            ARMARX_WARNING << "The entered path is NULL.";
        }
        std::filesystem::path p = _fullPath(*p_ptr);
        if (!std::filesystem::is_directory(p))
        {
            ARMARX_ERROR << "The entered path is not leading to a file! This is an error since if the folder for an EntityInstance exists there must be a data file in it (containing at least the metadata).";
        }
        else
        {
            path = p_ptr;
        }
    }

    void EntityInstance::setTo(const wm::EntityInstance& m)
    {
        std::filesystem::path p = _fullPath();

        try
        {
            std::filesystem::create_directories(p);
        }
        catch (...)
        {
            ARMARX_WARNING << GetHandledExceptionString();
            return;
        }

        std::filesystem::path d = p / (std::string(DATA_FILENAME) + ".json");

        if (std::filesystem::is_regular_file(d))
        {
            std::filesystem::remove(d);
        }

        std::ofstream ofs;
        ofs.open(d);

        auto aron = std::make_shared<aron::datanavigator::DictNavigator>();
        to_aron(aron, m);
        nlohmann::json j;
        from_aron(aron, j);

        ofs << j.dump(2);
        ofs.close();
    }
}
