#pragma once

#include <filesystem>

#include <RobotAPI/libraries/armem/core/base/EntityInstanceBase.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>

#include <RobotAPI/libraries/aron/core/navigator/type/forward_declarations.h>


namespace armarx::armem::d_ltm
{
    /**
     * @brief Data of a single entity instance.
     */
    class EntityInstance :
        public base::EntityInstanceBase<base::NoData, base::NoData>
    {
        using Base = base::EntityInstanceBase<base::NoData, base::NoData>;

    public:

        using Base::EntityInstanceBase;


        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        void update(const EntityUpdate& update);

        bool equalsDeep(const EntityInstance& other) const;


        // Conversion
        wm::EntityInstance convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const;

        // Filesystem connection
        void reload(const std::shared_ptr<std::filesystem::path>&);
        void setTo(const wm::EntityInstance&);


    private:
        std::filesystem::path _fullPath() const;
        std::filesystem::path _fullPath(const std::filesystem::path&) const;

    public:
        std::shared_ptr<std::filesystem::path> path;

    private:
        static const constexpr char* DATA_FILENAME = "data";
    };
}
