#include "Memory.h"

#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "error.h"


namespace armarx::armem::d_ltm
{
    std::filesystem::path Memory::_fullPath() const
    {
        if (path)
        {
            return _fullPath(*path);
        }
        return std::filesystem::path();
    }

    std::filesystem::path Memory::_fullPath(const std::filesystem::path& path) const
    {
        return path / id().memoryName;
    }

    wm::Memory Memory::convert() const
    {
        wm::Memory m(id());
        for (const auto& [k, s] : _container)
        {
            m.addCoreSegment(s.convert());
        }
        return m;
    }

    void Memory::reload(const std::filesystem::path& p)
    {
        if (std::filesystem::is_regular_file(p))
        {
            ARMARX_ERROR << "The entered path is leading to a file! Abort due to error.";
        }

        _container.clear();
        path = std::make_shared<std::filesystem::path>(p.parent_path());

        if (!std::filesystem::exists(p))
        {
            ARMARX_INFO << "The entered path does not exist. Returning empty memory.";
        }
        else
        {
            id() = MemoryID().withMemoryName(p.filename());

            for (const auto& d : std::filesystem::directory_iterator(p))
            {
                if (d.is_directory())
                {
                    std::string k = d.path().filename();
                    auto wms = _container.emplace(k, id().withCoreSegmentName(k));
                    wms.first->second.reload(path);
                }
            }
        }
    }

    void Memory::append(const wm::Memory& m)
    {
        std::filesystem::create_directories(_fullPath());
        m.forEachCoreSegment([this](const wm::CoreSegment & s) -> bool
        {
            if (auto it = _container.find(s.name()); it != _container.end())
            {
                it->second.append(s);
            }
            else
            {
                try
                {
                    std::filesystem::create_directory(_fullPath() / s.name());
                }
                catch (...)
                {
                    ARMARX_WARNING << GetHandledExceptionString();
                    return false;
                }

                auto wms = _container.emplace(s.name(), id().withCoreSegmentName(s.name()));
                wms.first->second.path = path;
                wms.first->second.append(s);
            }

            return true;
        });
    }
}
