#include "ProviderSegment.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "TypeIO.h"


namespace armarx::armem::d_ltm
{
    std::filesystem::path ProviderSegment::_fullPath() const
    {
        if (path)
        {
            return _fullPath(*path);
        }
        return std::filesystem::path();
    }

    std::filesystem::path ProviderSegment::_fullPath(const std::filesystem::path& path) const
    {
        return path / id().memoryName / id().coreSegmentName / id().providerSegmentName;
    }

    wm::ProviderSegment ProviderSegment::convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const
    {
        wm::ProviderSegment m(id());
        for (const auto& [_, s] : _container)
        {
            if (hasAronType())
            {
                m.addEntity(s.convert(_aronType));
            }
            else
            {
                m.addEntity(s.convert(expectedStructure));
            }
        }
        return m;
    }

    void ProviderSegment::reload(const std::shared_ptr<std::filesystem::path>& p_ptr)
    {
        if (!p_ptr)
        {
            ARMARX_WARNING << "The entered is NULL.";
        }
        std::filesystem::path p = _fullPath(*p_ptr);
        if (std::filesystem::is_regular_file(p))
        {
            ARMARX_ERROR << "The entered path is leading to a file! Abort due to error.";
        }

        _container.clear();
        path = p_ptr;

        if (!std::filesystem::exists(p))
        {
            ARMARX_INFO << "The entered path does not exist. Assuming an empty container.";
        }
        else
        {
            for (const auto& d : std::filesystem::directory_iterator(p))
            {
                if (d.is_directory())
                {
                    std::string k = d.path().filename();
                    auto wms = _container.emplace(k, id().withEntityName(k));
                    wms.first->second.reload(p_ptr);
                }

                if (d.is_regular_file())
                {
                    if (auto type = TypeIO::readAronType(d.path()))
                    {
                        _aronType = type;
                    }
                }
            }
        }
    }

    void ProviderSegment::append(const wm::ProviderSegment& m)
    {

        try
        {
            std::filesystem::create_directories(_fullPath());

        }
        catch (...)
        {
            ARMARX_WARNING << GetHandledExceptionString();
            return;
        }

        TypeIO::writeAronType(_aronType, _fullPath());

        m.forEachEntity([this](const wm::Entity & s)
        {
            if (auto it = _container.find(s.name()); it != _container.end())
            {
                it->second.append(s);
            }
            else
            {

                try
                {
                    std::filesystem::create_directory(_fullPath() / s.name());
                }
                catch (...)
                {
                    ARMARX_WARNING << GetHandledExceptionString();
                    return false;
                }

                auto wms = _container.emplace(s.name(), id().withEntityName(s.name()));
                wms.first->second.path = path;
                wms.first->second.append(s);
            }

            return true;
        });
    }
}
