#include "CoreSegment.h"

#include "error.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <SimoxUtility/json/json.hpp>


namespace armarx::armem::ltm
{

    wm::CoreSegment CoreSegment::convert() const
    {
        wm::CoreSegment m(id());
        for (const auto& [_, s] : _container)
        {
            m.addProviderSegment(s.convert());
        }
        return m;
    }

    void CoreSegment::reload()
    {
        _container.clear();

        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().str()];


        mongocxx::cursor cursor = coll.find({});
        for (auto doc : cursor)
        {
            nlohmann::json json = nlohmann::json::parse(bsoncxx::to_json(doc));
            ARMARX_INFO << "CoreSegment: Found foreign key: " << json.at("foreign_key");

            MemoryID i = MemoryID::fromString(json.at("foreign_key").get<std::string>());
            if (i.coreSegmentName != id().coreSegmentName)
            {
                throw error::InvalidMemoryID(i, "A MemoryID in mongodb was invalid. Found the wrong coreSegment name. Expected " + id().coreSegmentName);
            }

            std::string k = i.providerSegmentName;

            if (auto it = _container.find(k); it != _container.end())
            {
                throw error::ArMemError("Somehow after clearing the (core) container a key k = " + k + " was found. Do you have double entries in mongodb?");
            }
            else
            {
                auto wms = _container.emplace(k, id().withProviderSegmentName(k));
                wms.first->second.dbsettings = dbsettings;
                wms.first->second.reload();
            }
        }

        ARMARX_INFO << "After reload has core segment " << id().str() << " size: " << _container.size();
    }

    void CoreSegment::append(const wm::CoreSegment& m)
    {
        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().str()];

        m.forEachProviderSegment([this, &coll](const wm::ProviderSegment & provSeg)
        {
            auto it = _container.find(provSeg.name());
            if (it == _container.end())
            {
                bsoncxx::builder::stream::document builder;
                bsoncxx::document::value foreign_key = builder
                                                       << "foreign_key" << provSeg.id().str()
                                                       << bsoncxx::builder::stream::finalize;
                coll.insert_one(foreign_key.view());

                it = _container.emplace(provSeg.name(), id().withProviderSegmentName(provSeg.name())).first;
                it->second.dbsettings = dbsettings;
            }
            it->second.append(provSeg);

            return true;
        });
    }
}
