#pragma once

#include "EntitySnapshot.h"
#include "mongodb/MongoDBConnectionManager.h"

#include <RobotAPI/libraries/armem/core/base/EntityBase.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>


namespace armarx::armem::ltm
{
    /**
     * @brief An entity over a period of time.
     *
     * An entity should be a physical thing or abstract concept existing
     * (and potentially evolving) over some time.
     *
     * Examples are:
     * - objects (the green box)
     * - agents (robot, human)
     * - locations (frige, sink)
     * - grasp affordances (general, or for a specific object)
     * - images
     * - point clouds
     * - other sensory values
     *
     * At each point in time (`EntitySnapshot`), the entity can have a
     * (potentially variable) number of instances (`EntityInstance`),
     * each containing a single `AronData` object of a specific `AronType`.
     */
    class Entity :
        public base::EntityBase<EntitySnapshot, Entity>
    {
        using Base = base::EntityBase<EntitySnapshot, Entity>;

    public:

        using Base::EntityBase;


        // Conversion
        wm::Entity convert() const;

        // MongoDB connection
        void reload();
        void append(const wm::Entity&);

        // overrides for LTM lookups
        bool hasSnapshot(const Time& time) const;
        std::vector<Time> getTimestamps() const;
        const EntitySnapshot& getSnapshot(const Time& time) const;


    protected:
        // overrides for LTM storage
        const ContainerT::value_type& getLatestItem() const;

    public:
        MongoDBConnectionManager::MongoDBSettings dbsettings;

    };

}
