#include "EntitySnapshot.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/libraries/aron/core/navigator/data/container/Dict.h>

#include <RobotAPI/libraries/armem/core/wm/aron_conversions.h>
#include <RobotAPI/libraries/armem/core/wm/json_conversions.h>


#include "error.h"


namespace armarx::armem::ltm
{

    wm::EntitySnapshot EntitySnapshot::convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const
    {
        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().getEntityID().str()];

        auto res = coll.find_one(document{} << "id" << id().getEntitySnapshotID().str() << finalize);
        if (!res)
        {
            throw error::ArMemError("Could not load an instance from the memory '" + id().getEntityID().str() + "'. Tried to access: " + id().getEntitySnapshotID().str());
        }

        nlohmann::json json = nlohmann::json::parse(bsoncxx::to_json(*res));
        nlohmann::json instances = json["instances"];

        if (instances.size() != _container.size())
        {
            throw error::ArMemError("The size of the mongodb entity entry at id " + id().getEntitySnapshotID().str() + " has wrong size. Expected: " + std::to_string(_container.size()) + " but got: " + std::to_string(instances.size()));
        }

        wm::EntitySnapshot m(id());
        for (unsigned int i = 0; i < _container.size(); ++i)
        {
            nlohmann::json doc = instances[i++];

            auto aron = std::make_shared<aron::datanavigator::DictNavigator>();
            to_aron(aron, doc, expectedStructure);
            wm::EntityInstance e(id());
            from_aron(aron, e);
            m.addInstance(e);
        }
        return m;
    }

    void EntitySnapshot::reload()
    {
        _container.clear();

        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().getEntityID().str()];

        auto res = coll.find_one(document{} << "id" << id().getEntitySnapshotID().str() << finalize);

        if (!res)
        {
            throw error::ArMemError("Could not load an instance from the memory '" + id().getEntityID().str() + "'. Tried to access: " + id().getEntitySnapshotID().str());
        }

        nlohmann::json json = nlohmann::json::parse(bsoncxx::to_json(*res));
        for (unsigned int i = 0; i < json.at("instances").size(); ++i)
        {
            _container.emplace_back(id().withInstanceIndex(i));
        }
    }

    void EntitySnapshot::setTo(const wm::EntitySnapshot& m, const armem::Time& t)
    {
        // We remove the contente here and reset it with new values
        _container.clear();

        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().getEntityID().str()];

        bsoncxx::builder::stream::document builder{};
        auto in_array = builder
                        << "id" << id().getEntitySnapshotID().str()
                        << "timestamp" << t.toMicroSeconds()
                        << "instances";
        auto array_builder = bsoncxx::builder::basic::array{};

        int i = 0;
        m.forEachInstance([this, &array_builder, &i](const wm::EntityInstance & instance)
        {
            auto wms = _container.emplace_back(id().withInstanceIndex(i++));

            auto aron = std::make_shared<aron::datanavigator::DictNavigator>();
            to_aron(aron, instance);
            nlohmann::json j;
            from_aron(aron, j);

            auto doc_value = bsoncxx::from_json(j.dump(2));
            array_builder.append(doc_value);

            return true;
        });

        auto after_array = in_array << array_builder;
        bsoncxx::document::value doc = after_array << bsoncxx::builder::stream::finalize;
        coll.insert_one(doc.view());
    }
}
