#pragma once

#include "CoreSegment.h"
#include "mongodb/MongoDBConnectionManager.h"

#include <RobotAPI/libraries/armem/core/base/MemoryBase.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>

#include <ArmarXCore/core/application/properties/forward_declarations.h>

#include <mutex>


namespace armarx::armem::ltm
{
    /**
     * @brief Data of a memory consisting of multiple core segments.
     */
    class Memory :
        public base::MemoryBase<CoreSegment, Memory>
    {
        using Base = base::MemoryBase<CoreSegment, Memory>;

    public:

        struct TransferSettings
        {
            bool enabled = false;
        };

        struct PeriodicTransferSettings : public TransferSettings
        {
            bool deleteFromWMOnTransfer = false;
            int frequencyHz = 1;
        };

        struct OnFullTransferSettings : public TransferSettings
        {
            enum class Mode
            {
                TRANSFER_LATEST,
                TRANSFER_LEAST_USED
            };

            Mode mode;
            int batch_size = 20;
        };


    public:

        using Base::MemoryBase;

        Memory(const Memory& other);
        Memory(Memory&& other);
        Memory& operator=(const Memory& other);
        Memory& operator=(Memory&& other);

        // PropertyDefinitions related to LTM
        void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "");

        // Conversion
        wm::Memory convert() const;

        // MongoDB connection
        void reload();
        void append(const wm::Memory&);


    private:
        bool checkConnection() const;

        void _append(const wm::Memory&);

    public:
        MongoDBConnectionManager::MongoDBSettings dbsettings;

        PeriodicTransferSettings periodicTransferSettings;
        OnFullTransferSettings onFullTransferSettings;

    private:
        bool reloaded = false;
        mutable std::mutex mongoDBMutex;

        std::vector<wm::Memory> toAppendQueue;
    };
}
