#include "ProviderSegment.h"

#include "error.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <SimoxUtility/json/json.hpp>


namespace armarx::armem::ltm
{

    wm::ProviderSegment ProviderSegment::convert() const
    {
        wm::ProviderSegment m(id());
        for (const auto& [_, s] : _container)
        {
            m.addEntity(s.convert());
        }
        return m;
    }

    void ProviderSegment::reload()
    {
        _container.clear();

        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().str()];

        mongocxx::cursor cursor = coll.find({});
        for (auto doc : cursor)
        {
            nlohmann::json json = nlohmann::json::parse(bsoncxx::to_json(doc));

            MemoryID i((std::string) json.at("foreign_key"));
            if (i.providerSegmentName != id().providerSegmentName)
            {
                throw error::InvalidMemoryID(i, "A MemoryID in mongodb was invalid. Found the wrong providerSegment name. Expected " + id().providerSegmentName);
            }

            std::string k = i.entityName;

            if (auto it = _container.find(k); it != _container.end())
            {
                throw error::ArMemError("Somehow after clearing the (provvider) container a key k = " + k + " was found. Do you have double entries in mongodb?");
            }
            else
            {
                auto wms = _container.emplace(k, id().withEntityName(k));
                wms.first->second.dbsettings = dbsettings;
                wms.first->second.reload();
            }
        }

        ARMARX_INFO << "After reload has provider segment " << id().str() << " size: " << _container.size();
    }

    void ProviderSegment::append(const wm::ProviderSegment& m)
    {
        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().str()];

        m.forEachEntity([this, &coll](const wm::Entity & s)
        {
            if (auto it = _container.find(s.name()); it != _container.end())
            {
                it->second.append(s);
            }
            else
            {
                auto builder = bsoncxx::builder::stream::document{};
                bsoncxx::document::value foreign_key = builder
                                                       << "foreign_key" << s.id().withEntityName(s.name()).str()
                                                       << bsoncxx::builder::stream::finalize;
                coll.insert_one(foreign_key.view());

                auto wms = _container.emplace(s.name(), id().withEntityName(s.name()));
                wms.first->second.dbsettings = dbsettings;
                wms.first->second.append(s);
            }

            return true;
        });
    }
}
