#include "memory_definitions.h"

#include "error.h"

#include <RobotAPI/libraries/aron/core/navigator/data/container/Dict.h>
#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <map>
#include <vector>


namespace armarx::armem::wm
{

    bool EntityInstance::equalsDeep(const EntityInstance& other) const
    {
        if (_data and other.data())
        {
            return id() == other.id()
                   && _metadata == other.metadata()
                   && *_data == *other.data();
        }
        if (_data or other.data())
        {
            return false;
        }
        return id() == other.id() && _metadata == other.metadata();
    }


    void EntityInstance::update(const EntityUpdate& update)
    {
        ARMARX_CHECK_FITS_SIZE(this->index(), update.instancesData.size());

        this->data() = update.instancesData.at(size_t(this->index()));

        this->_metadata.confidence = update.confidence;

        this->_metadata.timeCreated = update.timeCreated;
        this->_metadata.timeSent = update.timeSent;
        this->_metadata.timeArrived = update.timeArrived;
    }



    std::optional<wm::EntitySnapshot> CoreSegment::getLatestEntitySnapshot(const MemoryID& entityID) const
    {
        const wm::Entity& entity = this->getEntity(entityID);
        if (entity.empty())
        {
            return std::nullopt;
        }
        else
        {
            return entity.getLatestSnapshot();
        }
    }


    std::optional<wm::EntityInstance> CoreSegment::getLatestEntityInstance(
        const MemoryID& entityID, int instanceIndex) const
    {
        auto snapshot = getLatestEntitySnapshot(entityID);
        if (snapshot.has_value()
            and instanceIndex >= 0
            and static_cast<size_t>(instanceIndex) < snapshot->size())
        {
            return snapshot->getInstance(instanceIndex);
        }
        else
        {
            return std::nullopt;
        }
    }


    armarx::aron::datanavigator::DictNavigatorPtr
    CoreSegment::getLatestEntityInstanceData(const MemoryID& entityID, int instanceIndex) const
    {
        auto instance = getLatestEntityInstance(entityID, instanceIndex);
        if (instance.has_value())
        {
            return instance->data();
        }
        else
        {
            return nullptr;
        }
    }


}
