#pragma once

#include <RobotAPI/libraries/armem/core/base/EntityInstanceBase.h>
#include <RobotAPI/libraries/armem/core/base/EntitySnapshotBase.h>
#include <RobotAPI/libraries/armem/core/base/EntityBase.h>
#include <RobotAPI/libraries/armem/core/base/ProviderSegmentBase.h>
#include <RobotAPI/libraries/armem/core/base/CoreSegmentBase.h>
#include <RobotAPI/libraries/armem/core/base/MemoryBase.h>

#include <optional>


namespace armarx::armem::wm
{

    using EntityInstanceMetadata = base::EntityInstanceMetadata;
    using EntityInstanceData = armarx::aron::datanavigator::DictNavigator;
    using EntityInstanceDataPtr = armarx::aron::datanavigator::DictNavigatorPtr;


    /// @see base::EntityInstanceBase
    class EntityInstance :
        public base::EntityInstanceBase<EntityInstanceDataPtr, EntityInstanceMetadata>
    {
        using Base = base::EntityInstanceBase<EntityInstanceDataPtr, EntityInstanceMetadata>;

    public:

        using Base::EntityInstanceBase;


        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        void update(const EntityUpdate& update);

        bool equalsDeep(const EntityInstance& other) const;

    };



    /// @see base::EntitySnapshotBase
    class EntitySnapshot :
        public base::EntitySnapshotBase<EntityInstance, EntitySnapshot>
    {
    public:

        using base::EntitySnapshotBase<EntityInstance, EntitySnapshot>::EntitySnapshotBase;

    };


    /// @see base::EntityBase
    class Entity :
        public base::EntityBase<EntitySnapshot, Entity>
    {
    public:

        using base::EntityBase<EntitySnapshot, Entity>::EntityBase;

    };



    /// @see base::ProviderSegmentBase
    class ProviderSegment :
        public base::ProviderSegmentBase<Entity, ProviderSegment>
    {
    public:

        using base::ProviderSegmentBase<Entity, ProviderSegment>::ProviderSegmentBase;

    };



    /// @see base::CoreSegmentBase
    class CoreSegment :
        public base::CoreSegmentBase<ProviderSegment, CoreSegment>
    {
        using Base = base::CoreSegmentBase<ProviderSegment, CoreSegment>;

    public:

        using Base::CoreSegmentBase;


        // Non-locking interface

        std::optional<wm::EntitySnapshot> getLatestEntitySnapshot(
            const MemoryID& entityID) const;
        std::optional<wm::EntityInstance> getLatestEntityInstance(
            const MemoryID& entityID, int instanceIndex = 0) const;
        armarx::aron::datanavigator::DictNavigatorPtr getLatestEntityInstanceData(
            const MemoryID& entityID, int instanceIndex = 0) const;


        template <class AronDtoT>
        std::optional<AronDtoT> getLatestEntityInstanceDataAs(
            const MemoryID& entityID, int instanceIndex = 0) const
        {
            wm::EntityInstanceDataPtr data = getLatestEntityInstanceData(entityID, instanceIndex);
            if (data)
            {
                AronDtoT aron;
                aron.fromAron(data);
                return aron;
            }
            else
            {
                return std::nullopt;
            }
        }

    };



    /// @see base::MemoryBase
    class Memory :
        public base::MemoryBase<CoreSegment, Memory>
    {
        using Base = base::MemoryBase<CoreSegment, Memory>;

    public:

        using Base::MemoryBase;

    };

}

