#include "MemoryControlWidget.h"

#include <QPushButton>
#include <QLineEdit>
#include <QTimer>
#include <QHBoxLayout>
#include <QFileDialog>

#include <cmath>


namespace armarx::armem::gui
{

    MemoryControlWidget::MemoryControlWidget()
    {
        setSizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        auto vlayout = new QVBoxLayout();
        auto hlayout1 = new QHBoxLayout();
        auto hlayout2 = new QHBoxLayout();

        hlayout1->setContentsMargins(10, 2, 10, 2);
        hlayout2->setContentsMargins(10, 2, 10, 2);

        const int margin = 0;
        vlayout->setContentsMargins(margin, margin, margin, margin);

        _lineEdit = new QLineEdit("/tmp/MemoryExport", this);
        _lineEdit->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Preferred);
        _lineEdit->setMinimumWidth(150);

        _openFileBrowserButton = new QPushButton("Search files", this);

        _storeOnDiskButton = new QPushButton("Store query (Disk)", this);
        _storeInLTMButton = new QPushButton("Store query (LTM)", this);

        hlayout1->addWidget(_lineEdit);
        hlayout1->addWidget(_openFileBrowserButton);

        hlayout2->addWidget(_storeOnDiskButton);
        hlayout2->addWidget(_storeInLTMButton);

        vlayout->addItem(hlayout1);
        vlayout->addItem(hlayout2);

        this->setLayout(vlayout);
        // Private connections.

        // Public connections.
        connect(_openFileBrowserButton, &QPushButton::pressed, this, &This::openFileBrowser);
        connect(_storeInLTMButton, &QPushButton::pressed, this, &This::storeInLTM);
        connect(_storeOnDiskButton, &QPushButton::pressed, this, &This::storeOnDisk);
    }

    void MemoryControlWidget::openFileBrowser()
    {
        QFileDialog dialog;
        dialog.setFileMode(QFileDialog::DirectoryOnly);
        //dialog.setOption(QFileDialog::DontUseNativeDialog, true);
        dialog.setOption(QFileDialog::ShowDirsOnly, false);
        dialog.exec();
        QString open = dialog.directory().path();
        _lineEdit->setText(open);
    }

    QLineEdit* MemoryControlWidget::pathInputBox()
    {
        return _lineEdit;
    }

    QString MemoryControlWidget::getEnteredPath()
    {
        return _lineEdit->text();
    }

    QPushButton* MemoryControlWidget::storeInLTMButton()
    {
        return _storeInLTMButton;
    }

    QPushButton* MemoryControlWidget::storeOnDiskButton()
    {
        return _storeOnDiskButton;
    }
}

